/* 
 * Line follower
 * Prajescu Diana-Maria 331CC
 * Neculai Stefan 331CC
 * PM - proiect
 */

#ifndef F_CPU
#define F_CPU 16000000UL
#endif

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>

#define DELAY 	50
#define MED		55
#define SLOW	25
#define FAST	75	

/* set motor power */
void go(int speedLeft, int speedRight)
{
	/* left motor power */
	OCR1A=speedLeft;
	PORTC |= (1 << PC0);
	PORTC &=~(1 << PC1);
	
	/* right motor power */
	OCR1B=speedRight;
	PORTC |= (1 << PC2);
	PORTC &=~(1 << PC3);
}

/* stop motors */
void clear()
{
	PORTC &=~ ((1 << PC0) | (1 << PC1));
	PORTC &=~ ((1 << PC2) | (1 << PC3));
}

/* initializations */
void init()
{
	/* port C */
	DDRC = 0xFF;
	PORTC = 0x00;
	
	/* port A */
	DDRA = 0x00;
	PORTA = 0xFF;
	
	/* port D */
	DDRD = 0xFF;
	PORTD = 0x00;
	
	/* PWM */
	DDRD |= (1 <<PD4) | (1<<PD5);
	
	/* timer */
	TCCR1A |= (1<<COM1B1) |(1<<COM1A1) | (1<<WGM10) ;
	TCCR1B |=  (1<<CS12) | (1<<WGM12);
}

int main()
{
	int left, right;
	
	/* init */
	init();
	
	while(1)
	{
		left = 0;
		right = 0;
		
		/* left sensors */
		if(PINA & (1 << PINA0))
			left++;
		if(PINA & (1 << PINA1))
			left++;
		if(PINA & (1 << PINA2))
			left++;
		if(PINA & (1 << PINA3))
			left++;
			
		/* right sensors */
		if(PINA & (1 << PINA4))
			right++;
		if(PINA & (1 << PINA5))
			right++;
		if(PINA & (1 << PINA6))
			right++;
		if(PINA & (1 << PINA7))
			right++;
		

		/* middle */
		if ( right == left ){
			go(MED, MED);
		}

		/* too left */
		if ( right > left ){
			go(SLOW, FAST);
		}
			
		/* too right */
		if ( right < left ){
			go(FAST, SLOW);
		}
		
		/* wait */
		_delay_ms(DELAY);
		clear();
	}

	return 0;
}
